package de.c64_editor.charset;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;

import javax.swing.Icon;
import javax.swing.ImageIcon;

/**
 * Beschreibt einen C64-Character der Groesse 8x8.
 *
 * @author Argh The Awful
 */
public class C64Char {

	private byte[] character = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

	/**
	 * Konstruktor. Generiert ein neues Zeichen und gibt jeder Zeile den Wert 0.
	 */
	public C64Char() {
	}

	/**
	 * Konstruktor. Generiert aus einem byte-Array ein Zeichen. Jedes Zeile des
	 * Zeichens wird durch ein Byte des Arrays bestimmt. Ein Zeichen hat genau
	 * 8 Zeilen. Fehlende Zeilen werden durch 0 ersetzt, berzhlige Zeilen
	 * werden ignoriert.
	 *
	 * @param character byte-Array, aus dem das Zeichen generiert wird.
	 */
	public C64Char(byte[] character) {
		this.character = character;
	}

	/**
	 * Schreibt eine Line des Zeichens, reprsentiert durch einen byte-Wert.
	 *
	 * @param lineNumber Die Line des Zeichens, die berschrieben werden soll.
	 *                   Gltige Werte sind 0 bis 7.
	 * @param line Byte, das eine Linie des Zeichens reprsentiert.
	 * @return <code>true</code> falls eine gltige Zeile angegeben wurde und
	 *                           die Zeile berschrieben wurde,
	 *         <code>false</code> falls nicht.
	 */
	public boolean setLine(int lineNumber, byte line) {
		if (lineNumber < 0 || lineNumber > 7) {
			return false;
		} else {
			character[lineNumber] = line;
			return true;
		}
	}

	/**
	 * Gibt ein Byte zurck, da genau eine Zeile des Zeichens reprsentiert.
	 *
	 * @param lineNumber Die Zeile, die zurckgegeben werrden soll. Gltige
	 *                   Werte sind 0-7.
	 * @return Byte, da die entsprechende Zeile reprsentiert.
	 */
	public byte getLine(int lineNumber) {
		if (lineNumber < 0 || lineNumber > 7) {
			return 0x00;
		} else {
			return character[lineNumber];
		}
	}

	/**
	 * berschreibt alle 8 Bytes, aus denen sich das Zeichen zusammensetzt.
	 *
	 * @param character byte-Array, der das Zeichen reprsentiert. Der Array
	 *                  muss genau 8 Bytes lang sein, ansonsten wird das Zeichen
	 *                  nicht gesetzt.
	 * return <code>true</code> falls das Array 8 Bytes lang war, <BR>
	 * 		  <code>false</code> falls das Array nicht 8 Bytes lang war.
	 */
	public boolean setChar(byte[] character) {
		if (character.length == 8) {
			this.character = character;
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Gibt das Zeichen als 8 Byte langen Array vom Typ byte zurck.
	 *
	 * @return byte-Array, der das Zeichen reprsentiert. 
	 */
	public byte[] getChar() {
		return this.character;
	}

	/**
	 * Generiert aus einem {@link C64Char} ein {@link Icon}, da der Darstellung
	 * dieses Zeichens auf dem C64 im standard Text-Modus (8x8 Pixel bei zwei
	 * Farben). Die Vordergrundfarbe ist weiss, die Hintergrundfarbe schwarz.
	 *
	 * @param scale Faktor, um den das Bild skaliert wird.
	 * @return das erzeugte Bild des Zeichens.
	 */
	public Icon getIcon(int scale) {
		return getIcon(scale, 0, 1);
	}

	/**
	 * Generiert aus einem {@link C64Char} ein {@link Icon}, da der Darstellung
	 * dieses Zeichens auf dem C64 im single color Text-Modus (8x8 Pixel bei
	 * zwei Farben).
	 *
	 * @param scale Faktor, um den das Bild skaliert wird.
	 * @param d021 Wert des VIC-II-Registers $D021
	 * @param colorRamValue Zugehriger Wert des Color-RAMs.
	 * @return das erzeugte Bild des Zeichens.
	 */
	public Icon getIcon(int scale, int d021, int colorRamValue) {
		Image image = new BufferedImage(8 * scale, 8 * scale,
										BufferedImage.TYPE_INT_RGB);
		Graphics g = image.getGraphics();
		for (int y = 0; y < 8; y++) {
			byte line = this.getLine(y);
			for (int x = 0 ; x < 8; x++) {
				switch (line & 0x01) {
					case 0:
						g.setColor(C64Colors.getColor(d021));
						break;
					case 1:
						g.setColor(C64Colors.getColor(colorRamValue));
						break;
					default:
						break;
				}
					g.fillRect((7 - x) * scale, y * scale, scale, scale);
				line = (byte) (line >> 1);
			}
		}
		return new ImageIcon(image);
	}

	/**
	 * Generiert aus einem {@link C64Char} ein {@link Icon}, da der Darstellung
	 * dieses Zeichens auf dem C64 im Multicolor Text-Modus (4x8 Pixel bei vier
	 * Farben) entspricht.
	 * In Abhngigkeit des Werts im Color-RAM wird entweder das Bild fr ein
	 * Multicolor-Zeichen oder eine Singlecolor-Zeichen generiert.
	 *
	 * @param scale Faktor, um den das Bild skaliert wird.
	 * @param d021 Wert des VIC-II-Registers $D021
	 * @param d022 Wert des VIC-II-Registers $D022
	 * @param d023 Wert des VIC-II-Registers $D023
	 * @param colorRamValue Zugehriger Wert des Color-RAMs.
	 * @return das erzeugte Bild des Zeichens.
	 */
	public Icon getMCIcon(int scale,
			              int d021, int d022, int d023, int colorRamValue) {
		// Die Werte der vier Farben sollten im Bereich von 0 bis 15 liegen.
		d021 = d021 & 0x0F;
		d022 = d022 & 0x0F;
		d023 = d023 & 0x0F;
		colorRamValue = colorRamValue & 0x0F;
		/*
		 * Entscheide, ob Zeichen als Single- oder Multicolor dargestellt wird:
		 * Wert des Color-RAMs 00-07 (d.h. Bit 3 = 0): Singlecolor
		 * Wert des Color-RAMs 08-15 (d.h. Bit 3 = 1): Multicolor
		 */
		if (colorRamValue < 8) {
			// Singlecolor
			return getIcon(scale, d021, colorRamValue);
		} else {
			//Multicolor
			colorRamValue = colorRamValue - 8;
			Image image = new BufferedImage(8 * scale, 8 * scale,
											BufferedImage.TYPE_INT_RGB);
			Graphics g = image.getGraphics();
			for (int y = 0; y < 8; y++) {
				byte line = this.getLine(y);
				for (int x = 0; x < 4; x++) {
					switch (line & 0x03) {
						case 0:
							g.setColor(C64Colors.getColor(d021));
							break;
						case 1:
							g.setColor(C64Colors.getColor(d022));
							break;
						case 2:
							g.setColor(C64Colors.getColor(d023));
							break;
						case 3:
							g.setColor(C64Colors.getColor(colorRamValue));
							break;
						default:
							break;
					}
					g.fillRect((6 - x * 2) * scale, y * scale, scale * 2, scale);
					line = (byte) (line >> 2);
				}
			}
			return new ImageIcon(image);
		}
	}
}
