package de.c64_editor.charset;
/**
 * Representation eines Commodore 64 - CharSets bestehend aus 256 Zeichen.
 *
 * @author Argh The Awful
 */
public class C64CharSet {

	private C64Char[] charSet = new C64Char[256];

	/**
	 * Konstruktor. Bastelt einen Zeichensatz aus 256 Zeichen, wobei alle
	 * Zeichen aus default-Werten gebildet werden.
	 */
	public C64CharSet() {
		for (int i = 0; i < 256; i++) {
			charSet[i] = new C64Char();
		}
	}

	/**
	 * Bastelt aus einem Byte-Array ein C64-CharSet.
	 * Array muss mindestens 2048 Byte lang sein.
	 *
	 * @param byteArray
	 */
	public C64CharSet(byte byteArray[]) {
		setCharSet(byteArray);
	}

	/**
	 * Bastelt aus einem Char-Array ein C64-CharSet.
	 * Array muss mindestens 2048 Byte lang sein.
	 *
	 * @param charArray
	 */
	public C64CharSet(char charArray[]) {
		for (int i = 0; i < 256; i++) {
			C64Char c64Char = new C64Char();
			for (int j = 0; j < 8; j++) {
				c64Char.setLine(j, (byte)charArray[i * 8 + j]);
			}
			System.out.println();
			charSet[i] = c64Char;
		}
	}

	/**
	 * Ersetzt das gesamte Charset durch die in dem übergebenen Array übergebene
	 * Daten.
	 *
	 * @param byteArray Array mit den Daten, aus denen das neue Charset erstellt
	 *					werden soll. Das Array muss mindestens 2048 bytes lang
	 *					sein.
	 * @return <code>true</code> falls Charset ersetzt wurde,
	 *         <code>false</code> falls nicht.
	 */
	public boolean setCharSet(byte[] byteArray) {
		if (byteArray.length < 2048) {
			return false;
		} else {
			for (int i = 0; i < 256; i++) {
				C64Char c64Char = new C64Char();
				for (int j = 0; j < 8; j++) {
					c64Char.setLine(j, byteArray[i * 8 + j]);
				}
				charSet[i] = c64Char;
			}
			return true;
		}
	}

	/**
	 * Gibt ein Array vom Typ byte under der Länge 2048 zurück, daß die Daten,
	 * aus denen sich der Zeichensatz zusammensetzt, enthält.
	 *
	 * @return byte-Array mit den Zeichensatz-Daten.
	 */
	public byte[] getByteArray() {
		byte[] byteArray = new byte[2048];
		for (int i = 0; i < 256; i++) {
			C64Char c64Char = charSet[i];
			for (int j = 0; j < 8; j++) {
				byteArray[i * 8 + j] = c64Char.getLine(j);
			}
		}
		return byteArray;
	}

	/**
	 * Gibt ein Zeichen des Zeichensatzes zurueck.
	 *
	 * @param id Nummer des Zeichens.
	 * @return Das Zeichen.
	 */
	public C64Char getChar(int id) {
		return charSet[id];
	}

	/**
	 * Gibt ein Zeichen des Zeichensatzes zurueck.
	 *
	 * @param id Nummer des Zeichens.
	 * @return Das Zeichen.
	 */
	public C64Char getChar(byte id) {
		int i = id;
		if (i < 0) {
			i = i + 256;
		}
		return charSet[i];
	}

	/**
	 * Setzt ein Zeichen des Zeichensatzes.
	 *
	 * @param id Nummer des Zeichens.
	 * @param character Das Zeichen.
	 */
	public void setChar(int id, C64Char character) {
		charSet[id] = character;
	}
}
